// IeHelper.cs
// Inspired by Chad Brockman [chadb@chadb.net]

using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using mshtml;

namespace IeHelper
{
    public enum DOCHOSTUITYPE
    {
        DOCHOSTUITYPE_BROWSE = 0,
        DOCHOSTUITYPE_AUTHOR = 1
    }

    public enum DOCHOSTUIDBLCLK
    {
        DOCHOSTUIDBLCLK_DEFAULT = 0,
        DOCHOSTUIDBLCLK_SHOWPROPERTIES = 1,
        DOCHOSTUIDBLCLK_SHOWCODE = 2
    }

    public enum DOCHOSTUIFLAG
    {
        DOCHOSTUIFLAG_DIALOG = 0x00000001,
        DOCHOSTUIFLAG_DISABLE_HELP_MENU = 0x00000002,
        DOCHOSTUIFLAG_NO3DBORDER = 0x00000004,
        DOCHOSTUIFLAG_SCROLL_NO = 0x00000008,
        DOCHOSTUIFLAG_DISABLE_SCRIPT_INACTIVE = 0x00000010,
        DOCHOSTUIFLAG_OPENNEWWIN = 0x00000020,
        DOCHOSTUIFLAG_DISABLE_OFFSCREEN = 0x00000040,
        DOCHOSTUIFLAG_FLAT_SCROLLBAR = 0x00000080,
        DOCHOSTUIFLAG_DIV_BLOCKDEFAULT = 0x00000100,
        DOCHOSTUIFLAG_ACTIVATE_CLIENTHIT_ONLY = 0x00000200,
        DOCHOSTUIFLAG_OVERRIDEBEHAVIORFACTORY = 0x00000400,
        DOCHOSTUIFLAG_CODEPAGELINKEDFONTS = 0x00000800,
        DOCHOSTUIFLAG_URL_ENCODING_DISABLE_UTF8 = 0x00001000,
        DOCHOSTUIFLAG_URL_ENCODING_ENABLE_UTF8 = 0x00002000,
        DOCHOSTUIFLAG_ENABLE_FORMS_AUTOCOMPLETE = 0x00004000,
        DOCHOSTUIFLAG_ENABLE_INPLACE_NAVIGATION = 0x00010000,
        DOCHOSTUIFLAG_IME_ENABLE_RECONVERSION = 0x00020000,
        DOCHOSTUIFLAG_THEME = 0x00040000,
        DOCHOSTUIFLAG_NOTHEME = 0x00080000,
        DOCHOSTUIFLAG_NOPICS = 0x00100000,
        DOCHOSTUIFLAG_NO3DOUTERBORDER = 0x00200000,
        DOCHOSTUIFLAG_DELEGATESIDOFDISPATCH = 0x00400000
    }

    [ StructLayout( LayoutKind.Sequential )]
        public struct DOCHOSTUIINFO
    {
        uint cbSize;
        uint dwFlags;
        uint dwDoubleClick;
        [MarshalAs(UnmanagedType.BStr)] string pchHostCss;
        [MarshalAs(UnmanagedType.BStr)] string pchHostNS;
    }

    [StructLayout( LayoutKind.Sequential )]
        public struct tagMSG
    {
        IntPtr hwnd;
        int lParam;
        uint message;
        tagPOINT pt;
        uint time;
        uint wParam;
    }

    [ComImport(),
     InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
     GuidAttribute("bd3f23c0-d43e-11cf-893b-00aa00bdce1a")]
    public interface IDocHostUIHandler
    {
        [PreserveSig]
        uint ShowContextMenu(uint dwID, ref tagPOINT ppt,
            [MarshalAs(UnmanagedType.IUnknown)] object
            pcmdtReserved,
            [MarshalAs(UnmanagedType.IDispatch)]object
            pdispReserved);
        [PreserveSig]
        void GetHostInfo(ref DOCHOSTUIINFO pInfo);
        [PreserveSig]
        void ShowUI(uint dwID,
            [MarshalAs(UnmanagedType.Interface)]
            IntPtr
            pActiveObject,
            [MarshalAs(UnmanagedType.Interface)]
            IntPtr
            pCommandTarget,
            [MarshalAs(UnmanagedType.Interface)]
            IntPtr pFrame,
            [MarshalAs(UnmanagedType.Interface)] IntPtr
            pDoc);
        [PreserveSig]
        void HideUI();
        [PreserveSig]
        void UpdateUI();
        [PreserveSig]
        void EnableModeless(int fEnable);
        [PreserveSig]
        void OnDocWindowActivate(int fActivate);
        [PreserveSig]
        void OnFrameWindowActivate(int fActivate);
        [PreserveSig]
        void ResizeBorder(ref tagRECT prcBorder,
            [MarshalAs(UnmanagedType.Interface)] IntPtr
            pUIWindow, int
            fRameWindow);
        [PreserveSig]
        void TranslateAccelerator(ref tagMSG lpmsg, ref Guid
            pguidCmdGroup,
            uint nCmdID);
        [PreserveSig]
        void GetOptionKeyPath(ref string pchKey, uint dw);
        [PreserveSig]
        void
            GetDropTarget([MarshalAs(UnmanagedType.Interface)]
            IntPtr
            pDropTarget,
            [MarshalAs(UnmanagedType.Interface)]
            ref IntPtr
            ppDropTarget);
        [PreserveSig]
        void GetExternal([Out,
            MarshalAs(UnmanagedType.IDispatch)]
            out object ppDispatch);
        [PreserveSig]
        void TranslateUrl(uint dwTranslate, ref ushort
            pchURLIn,
            IntPtr
            ppchURLOut);
        [PreserveSig]
        void FilterDataObject(IDataObject pDO, ref IDataObject
            ppDORet);
    }

    [ComImport(),
     InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
     GuidAttribute("3050f3f0-98b5-11cf-bb82-00aa00bdce0b")]
    public interface ICustomDoc
    {
        [PreserveSig]
        void SetUIHandler(IDocHostUIHandler pUIHandler);
    }

    public class DocHostUIHandler : IDocHostUIHandler
    {
        #region Implementation of IDocHostUIHandler
        public uint ShowContextMenu(uint dwID, ref mshtml.tagPOINT ppt, object pcmdtReserved, object pdispReserved)
        {
            return 0;
        }

        public void GetHostInfo(ref IeHelper.DOCHOSTUIINFO pInfo)
        {
        }

        public void ShowUI(uint dwID, System.IntPtr pActiveObject, System.IntPtr pCommandTarget, System.IntPtr pFrame, System.IntPtr pDoc)
        {
        }

        public void HideUI()
        {
        }

        public void UpdateUI()
        {
        }

        public void EnableModeless(int fEnable)
        {
        }

        public void OnDocWindowActivate(int fActivate)
        {
        }

        public void OnFrameWindowActivate(int fActivate)
        {
        }

        public void ResizeBorder(ref mshtml.tagRECT prcBorder, System.IntPtr pUIWindow, int fRameWindow)
        {
        }

        public void TranslateAccelerator(ref IeHelper.tagMSG lpmsg, ref System.Guid pguidCmdGroup, uint nCmdID)
        {
        }

        public void GetOptionKeyPath(ref string pchKey, uint dw)
        {
        }

        public void GetDropTarget(System.IntPtr pDropTarget, ref System.IntPtr ppDropTarget)
        {
        }

        public void GetExternal(out object ppDispatch)
        {
            ppDispatch = externalDispatch;
        }

        public void TranslateUrl(uint dwTranslate, ref ushort pchURLIn, System.IntPtr ppchURLOut)
        {
        }

        public void FilterDataObject(System.Windows.Forms.IDataObject pDO, ref System.Windows.Forms.IDataObject ppDORet)
        {
        }

        #endregion

        object  externalDispatch;
        public object ExternalDispatch
        {
            get { return externalDispatch; }
            set { externalDispatch = value; }
        }

        public void SetHandler(AxSHDocVw.AxWebBrowser browser)
        {
            if( !browser.ReadyState.Equals(SHDocVw.tagREADYSTATE.READYSTATE_COMPLETE) )
            {
                throw new ApplicationException("browser not ready");
            }

            IHTMLDocument2 doc = (IHTMLDocument2)browser.Document;
            // set the UI handler of the document to this form through the ICustomDoc interface
            ((IeHelper.ICustomDoc)doc).SetUIHandler(this);
        }    
    }
}
